@@ -5,14 +5,15 @@
  *    involving overwritten, double freeing memory and loss of memory.
  *
  * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-1999 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
  * This code contributed by Karl Lehenbauer and Mark Diekhans
  *
- * SCCS: @(#) tclCkalloc.c 1.28 97/04/30 12:09:04
+ * RCS: @(#) $Id: tclCkalloc.c,v 1.5.4.1 1999/09/22 04:12:45 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -101,16 +102,57 @@ static int  init_malloced_bodies = TRUE;
     static int  validate_memory = FALSE;
 #endif
 
+/*
+ * The following variable indicates to TclFinalizeMemorySubsystem() 
+ * that it should dump out the state of memory before exiting.  If the
+ * value is non-NULL, it gives the name of the file in which to
+ * dump memory usage information.
+ */
+
+char *tclMemDumpFileName = NULL;
+
+static char dumpFile[100];	/* Records where to dump memory allocation
+				 * information. */
+
+/*
+ * Mutex to serialize allocations.  This is a low-level mutex that must
+ * be explicitly initialized.  This is necessary because the self
+ * initializing mutexes use ckalloc...
+ */
+static Tcl_Mutex *ckallocMutexPtr;
+static int ckallocInit = 0;
+
 /*
  * Prototypes for procedures defined in this file:
  */
 
+static int		CheckmemCmd _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Interp *interp, int argc, char *argv[]));
 static int		MemoryCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		ValidateMemory _ANSI_ARGS_((
 			    struct mem_header *memHeaderP, char *file,
 			    int line, int nukeGuards));
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclInitDbCkalloc --
+ *	Initialize the locks used by the allocator.
+ *	This is only appropriate to call in a single threaded environment,
+ *	such as during TclInitSubsystems.
+ *
+ *----------------------------------------------------------------------
+ */
+void
+TclInitDbCkalloc() 
+{
+    if (!ckallocInit) {
+	ckallocInit = 1;
+	ckallocMutexPtr = Tcl_GetAllocMutex();
+    }
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -164,7 +206,7 @@ ValidateMemory(memHeaderP, file, line, nukeGuards)
             fflush(stdout);
 	    byte &= 0xff;
             fprintf(stderr, "low guard byte %d is 0x%x  \t%c\n", idx, byte,
-		    (isprint(UCHAR(byte)) ? byte : ' '));
+		    (isprint(UCHAR(byte)) ? byte : ' ')); /* INTL: bytes */
         }
     }
     if (guard_failed) {
@@ -185,7 +227,7 @@ ValidateMemory(memHeaderP, file, line, nukeGuards)
             fflush (stdout);
 	    byte &= 0xff;
             fprintf(stderr, "hi guard byte %d is 0x%x  \t%c\n", idx, byte,
-		    (isprint(UCHAR(byte)) ? byte : ' '));
+		    (isprint(UCHAR(byte)) ? byte : ' ')); /* INTL: bytes */
         }
     }
 
@@ -222,9 +264,14 @@ Tcl_ValidateAllMemory (file, line)
 {
     struct mem_header *memScanP;
 
-    for (memScanP = allocHead; memScanP != NULL; memScanP = memScanP->flink)
+    if (!ckallocInit) {
+	TclInitDbCkalloc();
+    }
+    Tcl_MutexLock(ckallocMutexPtr);
+    for (memScanP = allocHead; memScanP != NULL; memScanP = memScanP->flink) {
         ValidateMemory(memScanP, file, line, FALSE);
-
+    }
+    Tcl_MutexUnlock(ckallocMutexPtr);
 }
 
 /*
@@ -246,10 +293,16 @@ Tcl_DumpActiveMemory (fileName)
     struct mem_header *memScanP;
     char              *address;
 
-    fileP = fopen(fileName, "w");
-    if (fileP == NULL)
-        return TCL_ERROR;
+    if (fileName == NULL) {
+	fileP = stderr;
+    } else {
+	fileP = fopen(fileName, "w");
+	if (fileP == NULL) {
+	    return TCL_ERROR;
+	}
+    }
 
+    Tcl_MutexLock(ckallocMutexPtr);
     for (memScanP = allocHead; memScanP != NULL; memScanP = memScanP->flink) {
         address = &memScanP->body [0];
         fprintf(fileP, "%8lx - %8lx  %7ld @ %s %d %s",
@@ -259,7 +312,11 @@ Tcl_DumpActiveMemory (fileName)
 		 (memScanP->tagPtr == NULL) ? "" : memScanP->tagPtr->string);
 	(void) fputc('\n', fileP);
     }
-    fclose (fileP);
+    Tcl_MutexUnlock(ckallocMutexPtr);
+
+    if (fileP != stderr) {
+	fclose (fileP);
+    }
     return TCL_OK;
 }
 
@@ -313,6 +370,10 @@ Tcl_DbCkalloc(size, file, line)
 	memset ((char *) result->low_guard, GUARD_VALUE, LOW_GUARD_SIZE);
 	memset (result->body + size, GUARD_VALUE, HIGH_GUARD_SIZE);
     }
+    if (!ckallocInit) {
+	TclInitDbCkalloc();
+    }
+    Tcl_MutexLock(ckallocMutexPtr);
     result->length = size;
     result->tagPtr = curTagPtr;
     if (curTagPtr != NULL) {
@@ -322,6 +383,7 @@ Tcl_DbCkalloc(size, file, line)
     result->line = line;
     result->flink = allocHead;
     result->blink = NULL;
+
     if (allocHead != NULL)
         allocHead->blink = result;
     allocHead = result;
@@ -357,6 +419,8 @@ Tcl_DbCkalloc(size, file, line)
     if (current_bytes_malloced > maximum_bytes_malloced)
         maximum_bytes_malloced = current_bytes_malloced;
 
+    Tcl_MutexUnlock(ckallocMutexPtr);
+
     return result->body;
 }
 
@@ -381,10 +445,16 @@ Tcl_DbCkalloc(size, file, line)
 
 int
 Tcl_DbCkfree(ptr, file, line)
-    char *  ptr;
-    char     *file;
-    int       line;
+    char *ptr;
+    char *file;
+    int   line;
 {
+    struct mem_header *memp;
+
+    if (ptr == NULL) {
+ 	return;
+    }
+
     /*
      * The following cast is *very* tricky.  Must convert the pointer
      * to an integer before doing arithmetic on it, because otherwise
@@ -393,8 +463,7 @@ Tcl_DbCkfree(ptr, file, line)
      * even though BODY_OFFSET is in words on these machines).
      */
 
-    struct mem_header *memp = (struct mem_header *)
-	    (((unsigned long) ptr) - BODY_OFFSET);
+    memp = (struct mem_header *) (((unsigned long) ptr) - BODY_OFFSET);
 
     if (alloc_tracing)
         fprintf(stderr, "ckfree %lx %ld %s %d\n",
@@ -403,6 +472,7 @@ Tcl_DbCkfree(ptr, file, line)
     if (validate_memory)
         Tcl_ValidateAllMemory(file, line);
 
+    Tcl_MutexLock(ckallocMutexPtr);
     ValidateMemory(memp, file, line, TRUE);
     if (init_malloced_bodies) {
 	memset((VOID *) ptr, GUARD_VALUE, (size_t) memp->length);
@@ -429,6 +499,8 @@ Tcl_DbCkfree(ptr, file, line)
     if (allocHead == memp)
         allocHead = memp->flink;
     TclpFree((char *) memp);
+    Tcl_MutexUnlock(ckallocMutexPtr);
+
     return 0;
 }
 
@@ -453,14 +525,18 @@ Tcl_DbCkrealloc(ptr, size, file, line)
 {
     char *new;
     unsigned int copySize;
+    struct mem_header *memp;
+
+    if (ptr == NULL) {
+ 	return Tcl_DbCkalloc(size, file, line);
+    }
 
     /*
      * See comment from Tcl_DbCkfree before you change the following
      * line.
      */
 
-    struct mem_header *memp = (struct mem_header *)
-	    (((unsigned long) ptr) - BODY_OFFSET);
+    memp = (struct mem_header *) (((unsigned long) ptr) - BODY_OFFSET);
 
     copySize = size;
     if (copySize > (unsigned int) memp->length) {
@@ -580,7 +656,14 @@ MemoryCmd (clientData, interp, argc, argv)
         return TCL_OK;
     }
     if (strcmp(argv[1],"info") == 0) {
-        TclDumpMemoryInfo(stdout);
+	char buffer[400];
+	sprintf(buffer, "%-25s %10d\n%-25s %10d\n%-25s %10d\n%-25s %10d\n%-25s %10d\n%-25s %10d\n",
+	    "total mallocs", total_mallocs, "total frees", total_frees,
+	    "current packets allocated", current_malloc_packets,
+	    "current bytes allocated", current_bytes_malloced,
+	    "maximum packets allocated", maximum_malloc_packets,
+	    "maximum bytes allocated", maximum_bytes_malloced);
+	Tcl_SetResult(interp, buffer, TCL_VOLATILE);
         return TCL_OK;
     }
     if (strcmp(argv[1],"init") == 0) {
@@ -645,6 +728,42 @@ bad_suboption:
     return TCL_ERROR;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * CheckmemCmd --
+ *
+ *	This is the command procedure for the "checkmem" command, which
+ *	causes the application to exit after printing information about
+ *	memory usage to the file passed to this command as its first
+ *	argument.
+ *
+ * Results:
+ *	Returns a standard Tcl completion code.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+CheckmemCmd(clientData, interp, argc, argv)
+    ClientData clientData;		/* Not used. */
+    Tcl_Interp *interp;			/* Interpreter for evaluation. */
+    int argc;				/* Number of arguments. */
+    char *argv[];			/* String values of arguments. */
+{
+    if (argc != 2) {
+	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
+		" fileName\"", (char *) NULL);
+	return TCL_ERROR;
+    }
+    tclMemDumpFileName = dumpFile;
+    strcpy(tclMemDumpFileName, argv[1]);
+    return TCL_OK;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -657,11 +776,19 @@ void
 Tcl_InitMemory(interp)
     Tcl_Interp *interp;
 {
+    TclInitDbCkalloc();
     Tcl_CreateCommand (interp, "memory", MemoryCmd, (ClientData) NULL, 
 	    (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "checkmem", CheckmemCmd, (ClientData) 0,
+	    (Tcl_CmdDeleteProc *) NULL);
 }
 
-#else
+
+#else	/* TCL_MEM_DEBUG */
+
+#undef Tcl_InitMemory
+#undef Tcl_DumpActiveMemory
+#undef Tcl_ValidateAllMemory
 
 
 /*
@@ -678,12 +805,22 @@ char *
 Tcl_Alloc (size)
     unsigned int size;
 {
-        char *result;
+    char *result;
 
-        result = TclpAlloc(size);
-        if (result == NULL) 
-                panic("unable to alloc %d bytes", size);
-        return result;
+    result = TclpAlloc(size);
+    /*
+     * Most systems will not alloc(0), instead bumping it to one so
+     * that NULL isn't returned.  Some systems (AIX, Tru64) will alloc(0)
+     * by returning NULL, so we have to check that the NULL we get is
+     * not in response to alloc(0).
+     *
+     * The ANSI spec actually says that systems either return NULL *or*
+     * a special pointer on failure, but we only check for NULL
+     */
+    if ((result == NULL) && size) {
+	panic("unable to alloc %d bytes", size);
+    }
+    return result;
 }
 
 char *
@@ -696,7 +833,7 @@ Tcl_DbCkalloc(size, file, line)
 
     result = (char *) TclpAlloc(size);
 
-    if (result == NULL) {
+    if ((result == NULL) && size) {
         fflush(stdout);
         panic("unable to alloc %d bytes, %s line %d", size, file, 
               line);
@@ -723,8 +860,10 @@ Tcl_Realloc(ptr, size)
     char *result;
 
     result = TclpRealloc(ptr, size);
-    if (result == NULL) 
+
+    if ((result == NULL) && size) {
 	panic("unable to realloc %d bytes", size);
+    }
     return result;
 }
 
@@ -739,10 +878,9 @@ Tcl_DbCkrealloc(ptr, size, file, line)
 
     result = (char *) TclpRealloc(ptr, size);
 
-    if (result == NULL) {
+    if ((result == NULL) && size) {
         fflush(stdout);
-        panic("unable to realloc %d bytes, %s line %d", size, file, 
-              line);
+        panic("unable to realloc %d bytes, %s line %d", size, file, line);
     }
     return result;
 }
@@ -762,14 +900,14 @@ void
 Tcl_Free (ptr)
     char *ptr;
 {
-        TclpFree(ptr);
+    TclpFree(ptr);
 }
 
 int
 Tcl_DbCkfree(ptr, file, line)
-    char *  ptr;
-    char     *file;
-    int       line;
+    char *ptr;
+    char *file;
+    int   line;
 {
     TclpFree(ptr);
     return 0;
@@ -784,11 +922,6 @@ Tcl_DbCkfree(ptr, file, line)
  *
  *----------------------------------------------------------------------
  */
-
-#undef Tcl_InitMemory
-#undef Tcl_DumpActiveMemory
-#undef Tcl_ValidateAllMemory
-
 	/* ARGSUSED */
 void
 Tcl_InitMemory(interp)
@@ -810,4 +943,50 @@ Tcl_ValidateAllMemory(file, line)
 {
 }
 
+void
+TclDumpMemoryInfo(outFile) 
+    FILE *outFile;
+{
+}
+
+#endif	/* TCL_MEM_DEBUG */
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * TclFinalizeMemorySubsystem --
+ *
+ *	This procedure is called to finalize all the structures that 
+ *	are used by the memory allocator on a per-process basis.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	This subsystem is self-initializing, since memory can be 
+ *	allocated before Tcl is formally initialized.  After this call,
+ *	this subsystem has been reset to its initial state and is 
+ *	usable again.
+ *
+ *---------------------------------------------------------------------------
+ */
+
+void
+TclFinalizeMemorySubsystem()
+{
+#ifdef TCL_MEM_DEBUG
+    Tcl_MutexLock(ckallocMutexPtr);
+    if (tclMemDumpFileName != NULL) {
+	Tcl_DumpActiveMemory(tclMemDumpFileName);
+    }
+    if (curTagPtr != NULL) {
+	TclpFree((char *) curTagPtr);
+    }
+    allocHead = NULL;
+    Tcl_MutexUnlock(ckallocMutexPtr);
 #endif
+
+#if USE_TCLALLOC
+    TclFinalizeAllocSubsystem(); 
+#endif
+}
